@@ -1,8 +1,11 @@
+from __future__ import unicode_literals
+
 from django import forms
 from django.core.exceptions import ValidationError
 from django.core.validators import EMPTY_VALUES
 from django.utils.translation import ugettext_lazy as _
 
+from selectable.forms.base import import_lookup_class
 from selectable.forms.widgets import AutoCompleteSelectWidget
 from selectable.forms.widgets import AutoCompleteSelectMultipleWidget
 
@@ -16,11 +19,11 @@ class AutoCompleteSelectField(forms.Field):
     widget = AutoCompleteSelectWidget
 
     default_error_messages = {
-        'invalid_choice': _(u'Select a valid choice. That choice is not one of the available choices.'),
+        'invalid_choice': _('Select a valid choice. That choice is not one of the available choices.'),
     }
 
     def __init__(self, lookup_class, *args, **kwargs):
-        self.lookup_class = lookup_class
+        self.lookup_class = import_lookup_class(lookup_class)
         self.allow_new = kwargs.pop('allow_new', False)
         self.limit = kwargs.pop('limit', None)
         widget = kwargs.get('widget', self.widget) or self.widget
@@ -32,21 +35,24 @@ class AutoCompleteSelectField(forms.Field):
     def to_python(self, value):
         if value in EMPTY_VALUES:
             return None
-        lookup =self.lookup_class()
+        lookup = self.lookup_class()
         if isinstance(value, list):
             # Input comes from an AutoComplete widget. It's two
             # components: text and id
             if len(value) != 2:
                 raise ValidationError(self.error_messages['invalid_choice'])
-            if value[1] in EMPTY_VALUES:
+            label, pk = value
+            if pk in EMPTY_VALUES:
                 if not self.allow_new:
-                    if value[0]:
+                    if label:
                         raise ValidationError(self.error_messages['invalid_choice'])
                     else:
                         return None
-                value = lookup.create_item(value[0])  
+                if label in EMPTY_VALUES:
+                    return None
+                value = lookup.create_item(label)
             else:
-                value = lookup.get_item(value[1])
+                value = lookup.get_item(pk)
                 if value is None:
                     raise ValidationError(self.error_messages['invalid_choice'])
         else:
@@ -60,11 +66,11 @@ class AutoCompleteSelectMultipleField(forms.Field):
     widget = AutoCompleteSelectMultipleWidget
 
     default_error_messages = {
-        'invalid_choice': _(u'Select a valid choice. That choice is not one of the available choices.'),
+        'invalid_choice': _('Select a valid choice. That choice is not one of the available choices.'),
     }
 
     def __init__(self, lookup_class, *args, **kwargs):
-        self.lookup_class = lookup_class
+        self.lookup_class = import_lookup_class(lookup_class)
         self.limit = kwargs.pop('limit', None)
         widget = kwargs.get('widget', self.widget) or self.widget
         if isinstance(widget, type):
