@@ -1,10 +1,15 @@
+from __future__ import unicode_literals
+
+import json
+
 from django import forms
 from django.conf import settings
 from django.forms.util import flatatt
 from django.utils.http import urlencode
-from django.utils.encoding import force_unicode
 from django.utils.safestring import mark_safe
 
+from selectable.compat import force_text
+from selectable.forms.base import import_lookup_class
 
 __all__ = (
     'AutoCompleteWidget',
@@ -16,22 +21,22 @@ __all__ = (
 )
 
 
-STATIC_PREFIX = u'%sselectable/' % settings.STATIC_URL
+STATIC_PREFIX = '%sselectable/' % settings.STATIC_URL
 
 
 class SelectableMediaMixin(object):
 
     class Media(object):
         css = {
-            'all': (u'%scss/dj.selectable.css' % STATIC_PREFIX, )
+            'all': ('%scss/dj.selectable.css' % STATIC_PREFIX, )
         }
-        js = (u'%sjs/jquery.dj.selectable.js' % STATIC_PREFIX, )
+        js = ('%sjs/jquery.dj.selectable.js' % STATIC_PREFIX, )
 
 
 class AutoCompleteWidget(forms.TextInput, SelectableMediaMixin):
 
     def __init__(self, lookup_class, *args, **kwargs):
-        self.lookup_class = lookup_class
+        self.lookup_class = import_lookup_class(lookup_class)
         self.allow_new = kwargs.pop('allow_new', False)
         self.qs = kwargs.pop('query_params', {})
         self.limit = kwargs.pop('limit', None)
@@ -47,9 +52,11 @@ class AutoCompleteWidget(forms.TextInput, SelectableMediaMixin):
             self.qs['limit'] = self.limit
         if self.qs:
             url = '%s?%s' % (url, urlencode(self.qs))
-        attrs[u'data-selectable-url'] = url
-        attrs[u'data-selectable-type'] = 'text'
-        attrs[u'data-selectable-allow-new'] = str(self.allow_new).lower()
+        if 'data-selectable-options' in attrs:
+            attrs['data-selectable-options'] = json.dumps(attrs['data-selectable-options'])
+        attrs['data-selectable-url'] = url
+        attrs['data-selectable-type'] = 'text'
+        attrs['data-selectable-allow-new'] = str(self.allow_new).lower()
         return attrs
 
 
@@ -58,22 +65,13 @@ class SelectableMultiWidget(forms.MultiWidget):
     def update_query_parameters(self, qs_dict):
         self.widgets[0].update_query_parameters(qs_dict)
 
-
-class AutoCompleteSelectWidget(SelectableMultiWidget, SelectableMediaMixin):
-
-    def __init__(self, lookup_class, *args, **kwargs):
-        self.lookup_class = lookup_class
-        self.allow_new = kwargs.pop('allow_new', False)
-        self.limit = kwargs.pop('limit', None)
-        query_params = kwargs.pop('query_params', {})
-        widgets = [
-            AutoCompleteWidget(
-                lookup_class, allow_new=self.allow_new,
-                limit=self.limit, query_params=query_params
-            ),
-            forms.HiddenInput(attrs={u'data-selectable-type': 'hidden'})
-        ]
-        super(AutoCompleteSelectWidget, self).__init__(widgets, *args, **kwargs)
+    def _has_changed(self, initial, data):
+        "Decects if the widget was changed. This is removed in 1.6."
+        if initial is None and data is None:
+            return False
+        if data and not hasattr(data, '__iter__'):
+            data = self.decompress(data)
+        return super(SelectableMultiWidget, self)._has_changed(initial, data)
 
     def decompress(self, value):
         if value:
@@ -88,6 +86,24 @@ class AutoCompleteSelectWidget(SelectableMultiWidget, SelectableMediaMixin):
             return [item_value, value]
         return [None, None]
 
+
+class AutoCompleteSelectWidget(SelectableMultiWidget, SelectableMediaMixin):
+
+    def __init__(self, lookup_class, *args, **kwargs):
+        self.lookup_class = import_lookup_class(lookup_class)
+        self.allow_new = kwargs.pop('allow_new', False)
+        self.limit = kwargs.pop('limit', None)
+        query_params = kwargs.pop('query_params', {})
+        widgets = [
+            AutoCompleteWidget(
+                lookup_class, allow_new=self.allow_new,
+                limit=self.limit, query_params=query_params,
+                attrs=kwargs.get('attrs'),
+            ),
+            forms.HiddenInput(attrs={'data-selectable-type': 'hidden'})
+        ]
+        super(AutoCompleteSelectWidget, self).__init__(widgets, *args, **kwargs)
+
     def value_from_datadict(self, data, files, name):
         value = super(AutoCompleteSelectWidget, self).value_from_datadict(data, files, name)
         if not self.allow_new:
@@ -99,39 +115,27 @@ class AutoComboboxWidget(AutoCompleteWidget, SelectableMediaMixin):
 
     def build_attrs(self, extra_attrs=None, **kwargs):
         attrs = super(AutoComboboxWidget, self).build_attrs(extra_attrs, **kwargs)
-        attrs[u'data-selectable-type'] = 'combobox'
+        attrs['data-selectable-type'] = 'combobox'
         return attrs
 
 
 class AutoComboboxSelectWidget(SelectableMultiWidget, SelectableMediaMixin):
 
     def __init__(self, lookup_class, *args, **kwargs):
-        self.lookup_class = lookup_class
+        self.lookup_class = import_lookup_class(lookup_class)
         self.allow_new = kwargs.pop('allow_new', False)
         self.limit = kwargs.pop('limit', None)
         query_params = kwargs.pop('query_params', {})
         widgets = [
             AutoComboboxWidget(
                 lookup_class, allow_new=self.allow_new,
-                limit=self.limit, query_params=query_params
+                limit=self.limit, query_params=query_params,
+                attrs=kwargs.get('attrs'),
             ),
-            forms.HiddenInput(attrs={u'data-selectable-type': 'hidden'})
+            forms.HiddenInput(attrs={'data-selectable-type': 'hidden'})
         ]
         super(AutoComboboxSelectWidget, self).__init__(widgets, *args, **kwargs)
 
-    def decompress(self, value):
-        if value:
-            lookup = self.lookup_class()
-            model = getattr(self.lookup_class, 'model', None)
-            if model and isinstance(value, model):
-                item = value
-                value = lookup.get_item_id(item)
-            else:
-                item = lookup.get_item(value)
-            item_value = lookup.get_item_value(item)
-            return [item_value, value]
-        return [None, None]
-
     def value_from_datadict(self, data, files, name):
         value = super(AutoComboboxSelectWidget, self).value_from_datadict(data, files, name)
         if not self.allow_new:
@@ -142,7 +146,7 @@ class AutoComboboxSelectWidget(SelectableMultiWidget, SelectableMediaMixin):
 class LookupMultipleHiddenInput(forms.MultipleHiddenInput):
 
     def __init__(self, lookup_class, *args, **kwargs):
-        self.lookup_class = lookup_class
+        self.lookup_class = import_lookup_class(lookup_class)
         super(LookupMultipleHiddenInput, self).__init__(*args, **kwargs)
 
     def render(self, name, value, attrs=None, choices=()):
@@ -157,7 +161,7 @@ class LookupMultipleHiddenInput(forms.MultipleHiddenInput):
             if model and isinstance(v, model):
                 item = v
                 v = lookup.get_item_id(item)
-            input_attrs = dict(value=force_unicode(v), **final_attrs)
+            input_attrs = dict(value=force_text(v), **final_attrs)
             if id_:
                 # An ID attribute was given. Add a numeric index as a suffix
                 # so that the inputs don't all have the same ID attribute.
@@ -165,25 +169,26 @@ class LookupMultipleHiddenInput(forms.MultipleHiddenInput):
             if v:
                 item = item or lookup.get_item(v)
                 input_attrs['title'] = lookup.get_item_value(item)
-            inputs.append(u'<input%s />' % flatatt(input_attrs))
-        return mark_safe(u'\n'.join(inputs))
+            inputs.append('<input%s />' % flatatt(input_attrs))
+        return mark_safe('\n'.join(inputs))
 
     def build_attrs(self, extra_attrs=None, **kwargs):
         attrs = super(LookupMultipleHiddenInput, self).build_attrs(extra_attrs, **kwargs)
-        attrs[u'data-selectable-type'] = 'hidden-multiple'
+        attrs['data-selectable-type'] = 'hidden-multiple'
         return attrs
 
 
 class AutoCompleteSelectMultipleWidget(SelectableMultiWidget, SelectableMediaMixin):
 
     def __init__(self, lookup_class, *args, **kwargs):
-        self.lookup_class = lookup_class
+        self.lookup_class = import_lookup_class(lookup_class)
         self.limit = kwargs.pop('limit', None)
         position = kwargs.pop('position', 'bottom')
         attrs = {
-            u'data-selectable-multiple': 'true',
-            u'data-selectable-position': position
+            'data-selectable-multiple': 'true',
+            'data-selectable-position': position
         }
+        attrs.update(kwargs.get('attrs', {}))
         query_params = kwargs.pop('query_params', {})
         widgets = [
             AutoCompleteWidget(
@@ -200,20 +205,32 @@ class AutoCompleteSelectMultipleWidget(SelectableMultiWidget, SelectableMediaMix
     def render(self, name, value, attrs=None):
         if value and not hasattr(value, '__iter__'):
             value = [value]
-        value = [u'', value]
+        value = ['', value]
         return super(AutoCompleteSelectMultipleWidget, self).render(name, value, attrs)
 
+    def _has_changed(self, initial, data):
+        """"
+        Decects if the widget was changed. This is removed in 1.6.
+
+        For the multi-select case we only care if the hidden inputs changed.
+        """
+        initial = ['', initial]
+        data = ['', data]
+        return super(AutoCompleteSelectMultipleWidget, self)._has_changed(initial, data)
+
+
 
 class AutoComboboxSelectMultipleWidget(SelectableMultiWidget, SelectableMediaMixin):
 
     def __init__(self, lookup_class, *args, **kwargs):
-        self.lookup_class = lookup_class
+        self.lookup_class = import_lookup_class(lookup_class)
         self.limit = kwargs.pop('limit', None)
         position = kwargs.pop('position', 'bottom')
         attrs = {
-            u'data-selectable-multiple': 'true',
-            u'data-selectable-position': position
+            'data-selectable-multiple': 'true',
+            'data-selectable-position': position
         }
+        attrs.update(kwargs.get('attrs', {}))
         query_params = kwargs.pop('query_params', {})
         widgets = [
             AutoComboboxWidget(
@@ -230,6 +247,15 @@ class AutoComboboxSelectMultipleWidget(SelectableMultiWidget, SelectableMediaMix
     def render(self, name, value, attrs=None):
         if value and not hasattr(value, '__iter__'):
             value = [value]
-        value = [u'', value]
+        value = ['', value]
         return super(AutoComboboxSelectMultipleWidget, self).render(name, value, attrs)
 
+    def _has_changed(self, initial, data):
+        """"
+        Decects if the widget was changed. This is removed in 1.6.
+
+        For the multi-select case we only care if the hidden inputs changed.
+        """
+        initial = ['', initial]
+        data = ['', data]
+        return super(AutoComboboxSelectMultipleWidget, self)._has_changed(initial, data)
\ No newline at end of file
