@@ -1,5 +1,8 @@
+from __future__ import unicode_literals
+
 import random
 import string
+from xml.dom.minidom import parseString
 
 from django.conf import settings
 from django.core.urlresolvers import reverse
@@ -17,6 +20,23 @@ __all__ = (
 )
 
 
+def as_xml(html):
+    "Convert HTML portion to minidom node."
+    return parseString('<root>%s</root>' % html)
+
+
+def parsed_inputs(html):
+    "Returns a dictionary mapping name --> node of inputs found in the HTML."
+    node = as_xml(html)
+    inputs = {}
+    for field in node.getElementsByTagName('input'):
+        name = dict(field.attributes.items())['name']
+        current = inputs.get(name, [])
+        current.append(field)
+        inputs[name] = current
+    return inputs
+
+
 class PatchSettingsMixin(object):
     def setUp(self):
         super(PatchSettingsMixin, self).setUp()
@@ -35,7 +55,7 @@ class BaseSelectableTestCase(TestCase):
     urls = 'selectable.tests.urls'
 
     def get_random_string(self, length=10):
-        return u''.join(random.choice(string.ascii_letters) for x in xrange(length))
+        return ''.join(random.choice(string.ascii_letters) for x in range(length))
 
     def create_thing(self, data=None):
         data = data or {}
