@@ -1,11 +1,12 @@
-from urlparse import urlparse
+import json
 
 from django import forms
 from django.utils.http import urlencode
 
+from selectable.compat import urlparse
 from selectable.forms import widgets
 from selectable.tests import Thing, ThingLookup
-from selectable.tests.base import BaseSelectableTestCase
+from selectable.tests.base import BaseSelectableTestCase, parsed_inputs
 
 
 __all__ = (
@@ -29,6 +30,29 @@ class BaseWidgetTestCase(BaseSelectableTestCase):
         widget = self.get_widget_instance()
         self.assertEqual(widget.lookup_class, self.__class__.lookup_cls)
 
+    def test_dotted_path(self):
+        """
+        Ensure lookup_class can be imported from a dotted path.
+        """
+        dotted_path = '.'.join([self.__class__.lookup_cls.__module__, self.__class__.lookup_cls.__name__])
+        widget = self.__class__.widget_cls(dotted_path)
+        self.assertEqual(widget.lookup_class, self.__class__.lookup_cls)
+
+    def test_invalid_dotted_path(self):
+        """
+        An invalid lookup_class dotted path should raise an ImportError.
+        """
+        with self.assertRaises(ImportError):
+            self.__class__.widget_cls('this.is.an.invalid.path')
+
+    def test_dotted_path_wrong_type(self):
+        """
+        lookup_class must be a subclass of LookupBase.
+        """
+        dotted_path = 'selectable.forms.widgets.AutoCompleteWidget'
+        with self.assertRaises(TypeError):
+            self.__class__.widget_cls(dotted_path)
+
 
 class AutoCompleteWidgetTestCase(BaseWidgetTestCase):
     widget_cls = widgets.AutoCompleteWidget
@@ -42,7 +66,7 @@ class AutoCompleteWidgetTestCase(BaseWidgetTestCase):
         self.assertTrue('data-selectable-allow-new' in attrs)
 
     def test_update_query_parameters(self):
-        params = {'active': 1} 
+        params = {'active': 1}
         widget = self.get_widget_instance()
         widget.update_query_parameters(params)
         attrs = widget.build_attrs()
@@ -50,7 +74,7 @@ class AutoCompleteWidgetTestCase(BaseWidgetTestCase):
         parse = urlparse(url)
         query = parse.query
         self.assertEqual(query, urlencode(params))
-        
+
     def test_limit_paramter(self):
         widget = self.get_widget_instance(limit=10)
         attrs = widget.build_attrs()
@@ -60,7 +84,7 @@ class AutoCompleteWidgetTestCase(BaseWidgetTestCase):
         self.assertTrue('limit=10' in query)
 
     def test_initial_query_parameters(self):
-        params = {'active': 1} 
+        params = {'active': 1}
         widget = self.get_widget_instance(query_params=params)
         attrs = widget.build_attrs()
         url = attrs['data-selectable-url']
@@ -68,6 +92,14 @@ class AutoCompleteWidgetTestCase(BaseWidgetTestCase):
         query = parse.query
         self.assertEqual(query, urlencode(params))
 
+    def test_build_selectable_options(self):
+        "Serialize selectable options as json in data attribute."
+        options = {'autoFocus': True}
+        widget = self.get_widget_instance(attrs={'data-selectable-options': options})
+        attrs = widget.build_attrs()
+        self.assertTrue('data-selectable-options' in attrs)
+        self.assertEqual(attrs['data-selectable-options'], json.dumps(options))
+
 
 class AutoCompleteSelectWidgetTestCase(BaseWidgetTestCase):
     widget_cls = widgets.AutoCompleteSelectWidget
@@ -89,7 +121,7 @@ class AutoCompleteSelectWidgetTestCase(BaseWidgetTestCase):
         self.assertEqual(attrs['data-selectable-type'], 'hidden')
 
     def test_update_query_parameters(self):
-        params = {'active': 1} 
+        params = {'active': 1}
         widget = self.get_widget_instance()
         widget.update_query_parameters(params)
         sub_widget = widget.widgets[0]
@@ -109,7 +141,7 @@ class AutoCompleteSelectWidgetTestCase(BaseWidgetTestCase):
         self.assertTrue('limit=10' in query)
 
     def test_initial_query_parameters(self):
-        params = {'active': 1} 
+        params = {'active': 1}
         widget = self.get_widget_instance(query_params=params)
         sub_widget = widget.widgets[0]
         attrs = sub_widget.build_attrs()
@@ -118,6 +150,15 @@ class AutoCompleteSelectWidgetTestCase(BaseWidgetTestCase):
         query = parse.query
         self.assertEqual(query, urlencode(params))
 
+    def test_build_selectable_options(self):
+        "Serialize selectable options as json in data attribute."
+        options = {'autoFocus': True}
+        widget = self.get_widget_instance(attrs={'data-selectable-options': options})
+        sub_widget = widget.widgets[0]
+        attrs = sub_widget.build_attrs()
+        self.assertTrue('data-selectable-options' in attrs)
+        self.assertEqual(attrs['data-selectable-options'], json.dumps(options))
+
 
 class AutoComboboxWidgetTestCase(BaseWidgetTestCase):
     widget_cls = widgets.AutoComboboxWidget
@@ -131,7 +172,7 @@ class AutoComboboxWidgetTestCase(BaseWidgetTestCase):
         self.assertTrue('data-selectable-allow-new' in attrs)
 
     def test_update_query_parameters(self):
-        params = {'active': 1} 
+        params = {'active': 1}
         widget = self.get_widget_instance()
         widget.update_query_parameters(params)
         attrs = widget.build_attrs()
@@ -149,7 +190,7 @@ class AutoComboboxWidgetTestCase(BaseWidgetTestCase):
         self.assertTrue('limit=10' in query)
 
     def test_initial_query_parameters(self):
-        params = {'active': 1} 
+        params = {'active': 1}
         widget = self.get_widget_instance(query_params=params)
         attrs = widget.build_attrs()
         url = attrs['data-selectable-url']
@@ -157,6 +198,14 @@ class AutoComboboxWidgetTestCase(BaseWidgetTestCase):
         query = parse.query
         self.assertEqual(query, urlencode(params))
 
+    def test_build_selectable_options(self):
+        "Serialize selectable options as json in data attribute."
+        options = {'autoFocus': True}
+        widget = self.get_widget_instance(attrs={'data-selectable-options': options})
+        attrs = widget.build_attrs()
+        self.assertTrue('data-selectable-options' in attrs)
+        self.assertEqual(attrs['data-selectable-options'], json.dumps(options))
+
 
 class AutoComboboxSelectWidgetTestCase(BaseWidgetTestCase):
     widget_cls = widgets.AutoComboboxSelectWidget
@@ -178,7 +227,7 @@ class AutoComboboxSelectWidgetTestCase(BaseWidgetTestCase):
         self.assertEqual(attrs['data-selectable-type'], 'hidden')
 
     def test_update_query_parameters(self):
-        params = {'active': 1} 
+        params = {'active': 1}
         widget = self.get_widget_instance()
         widget.update_query_parameters(params)
         sub_widget = widget.widgets[0]
@@ -198,7 +247,7 @@ class AutoComboboxSelectWidgetTestCase(BaseWidgetTestCase):
         self.assertTrue('limit=10' in query)
 
     def test_initial_query_parameters(self):
-        params = {'active': 1} 
+        params = {'active': 1}
         widget = self.get_widget_instance(query_params=params)
         sub_widget = widget.widgets[0]
         attrs = sub_widget.build_attrs()
@@ -207,6 +256,15 @@ class AutoComboboxSelectWidgetTestCase(BaseWidgetTestCase):
         query = parse.query
         self.assertEqual(query, urlencode(params))
 
+    def test_build_selectable_options(self):
+        "Serialize selectable options as json in data attribute."
+        options = {'autoFocus': True}
+        widget = self.get_widget_instance(attrs={'data-selectable-options': options})
+        sub_widget = widget.widgets[0]
+        attrs = sub_widget.build_attrs()
+        self.assertTrue('data-selectable-options' in attrs)
+        self.assertEqual(attrs['data-selectable-options'], json.dumps(options))
+
 
 class AutoCompleteSelectMultipleWidgetTestCase(BaseWidgetTestCase):
     widget_cls = widgets.AutoCompleteSelectMultipleWidget
@@ -238,18 +296,23 @@ class AutoCompleteSelectMultipleWidgetTestCase(BaseWidgetTestCase):
         widget = self.get_widget_instance()
         val = 4
         rendered_value = widget.render('field_name', val)
-        ev = 'data-selectable-type="hidden-multiple" type="hidden" name="field_name_1" value="%d"' % val
-        self.assertTrue(ev in rendered_value,
-            "Did not find:\n\t%s\nin rendered value:\n\t%s" % (ev, rendered_value))
+        inputs = parsed_inputs(rendered_value)
+        field = inputs['field_name_1'][0]
+        self.assertEqual(field.attributes['data-selectable-type'].value, 'hidden-multiple')
+        self.assertEqual(field.attributes['type'].value, 'hidden')
+        self.assertEqual(int(field.attributes['value'].value), val)
 
     def test_render_list(self):
         widget = self.get_widget_instance()
         list_val = [8, 5]
         rendered_value = widget.render('field_name', list_val)
-        for v in list_val:
-            ev = 'data-selectable-type="hidden-multiple" type="hidden" name="field_name_1" value="%d"' % v
-            self.assertTrue(ev in rendered_value,
-                "Did not find:\n\t%s\nin rendered value:\n\t%s" % (ev, rendered_value))
+        inputs = parsed_inputs(rendered_value)
+        found_values = []
+        for field in inputs['field_name_1']:
+            self.assertEqual(field.attributes['data-selectable-type'].value, 'hidden-multiple')
+            self.assertEqual(field.attributes['type'].value, 'hidden')
+            found_values.append(int(field.attributes['value'].value))
+        self.assertListEqual(found_values, list_val)
 
     def test_render_qs(self):
         widget = self.get_widget_instance()
@@ -257,13 +320,16 @@ class AutoCompleteSelectMultipleWidgetTestCase(BaseWidgetTestCase):
         t2 = self.create_thing()
         qs_val = Thing.objects.filter(pk__in=[t1.pk, t2.pk]).values_list('pk', flat=True)
         rendered_value = widget.render('field_name', qs_val)
-        for t in qs_val:
-            ev = 'data-selectable-type="hidden-multiple" type="hidden" name="field_name_1" value="%d"' % t
-            self.assertTrue(ev in rendered_value,
-                "Did not find:\n\t%s\nin rendered value:\n\t%s" % (ev, rendered_value))
+        inputs = parsed_inputs(rendered_value)
+        found_values = []
+        for field in inputs['field_name_1']:
+            self.assertEqual(field.attributes['data-selectable-type'].value, 'hidden-multiple')
+            self.assertEqual(field.attributes['type'].value, 'hidden')
+            found_values.append(int(field.attributes['value'].value))
+        self.assertListEqual(found_values, [t1.pk, t2.pk])
 
     def test_update_query_parameters(self):
-        params = {'active': 1} 
+        params = {'active': 1}
         widget = self.get_widget_instance()
         widget.update_query_parameters(params)
         sub_widget = widget.widgets[0]
@@ -283,7 +349,7 @@ class AutoCompleteSelectMultipleWidgetTestCase(BaseWidgetTestCase):
         self.assertTrue('limit=10' in query)
 
     def test_initial_query_parameters(self):
-        params = {'active': 1} 
+        params = {'active': 1}
         widget = self.get_widget_instance(query_params=params)
         sub_widget = widget.widgets[0]
         attrs = sub_widget.build_attrs()
@@ -292,6 +358,15 @@ class AutoCompleteSelectMultipleWidgetTestCase(BaseWidgetTestCase):
         query = parse.query
         self.assertEqual(query, urlencode(params))
 
+    def test_build_selectable_options(self):
+        "Serialize selectable options as json in data attribute."
+        options = {'autoFocus': True}
+        widget = self.get_widget_instance(attrs={'data-selectable-options': options})
+        sub_widget = widget.widgets[0]
+        attrs = sub_widget.build_attrs()
+        self.assertTrue('data-selectable-options' in attrs)
+        self.assertEqual(attrs['data-selectable-options'], json.dumps(options))
+
 
 class AutoComboboxSelectMultipleWidgetTestCase(BaseWidgetTestCase):
     widget_cls = widgets.AutoComboboxSelectMultipleWidget
@@ -323,18 +398,23 @@ class AutoComboboxSelectMultipleWidgetTestCase(BaseWidgetTestCase):
         widget = self.get_widget_instance()
         val = 4
         rendered_value = widget.render('field_name', val)
-        ev = 'data-selectable-type="hidden-multiple" type="hidden" name="field_name_1" value="%d"' % val
-        self.assertTrue(ev in rendered_value,
-            "Did not find:\n\t%s\nin rendered value:\n\t%s" % (ev, rendered_value))
+        inputs = parsed_inputs(rendered_value)
+        field = inputs['field_name_1'][0]
+        self.assertEqual(field.attributes['data-selectable-type'].value, 'hidden-multiple')
+        self.assertEqual(field.attributes['type'].value, 'hidden')
+        self.assertEqual(field.attributes['value'].value, str(val))
 
     def test_render_list(self):
         widget = self.get_widget_instance()
         list_val = [8, 5]
         rendered_value = widget.render('field_name', list_val)
-        for v in list_val:
-            ev = 'data-selectable-type="hidden-multiple" type="hidden" name="field_name_1" value="%d"' % v
-            self.assertTrue(ev in rendered_value,
-                "Did not find:\n\t%s\nin rendered value:\n\t%s" % (ev, rendered_value))
+        inputs = parsed_inputs(rendered_value)
+        found_values = []
+        for field in inputs['field_name_1']:
+            self.assertEqual(field.attributes['data-selectable-type'].value, 'hidden-multiple')
+            self.assertEqual(field.attributes['type'].value, 'hidden')
+            found_values.append(int(field.attributes['value'].value))
+        self.assertListEqual(found_values, list_val)
 
     def test_render_qs(self):
         widget = self.get_widget_instance()
@@ -342,13 +422,16 @@ class AutoComboboxSelectMultipleWidgetTestCase(BaseWidgetTestCase):
         t2 = self.create_thing()
         qs_val = Thing.objects.filter(pk__in=[t1.pk, t2.pk]).values_list('pk', flat=True)
         rendered_value = widget.render('field_name', qs_val)
-        for t in qs_val:
-            ev = 'data-selectable-type="hidden-multiple" type="hidden" name="field_name_1" value="%d"' % t
-            self.assertTrue(ev in rendered_value,
-                "Did not find:\n\t%s\nin rendered value:\n\t%s" % (ev, rendered_value))
+        inputs = parsed_inputs(rendered_value)
+        found_values = []
+        for field in inputs['field_name_1']:
+            self.assertEqual(field.attributes['data-selectable-type'].value, 'hidden-multiple')
+            self.assertEqual(field.attributes['type'].value, 'hidden')
+            found_values.append(int(field.attributes['value'].value))
+        self.assertListEqual(found_values, [t1.pk, t2.pk])
 
     def test_update_query_parameters(self):
-        params = {'active': 1} 
+        params = {'active': 1}
         widget = self.get_widget_instance()
         widget.update_query_parameters(params)
         sub_widget = widget.widgets[0]
@@ -368,7 +451,7 @@ class AutoComboboxSelectMultipleWidgetTestCase(BaseWidgetTestCase):
         self.assertTrue('limit=10' in query)
 
     def test_initial_query_parameters(self):
-        params = {'active': 1} 
+        params = {'active': 1}
         widget = self.get_widget_instance(query_params=params)
         sub_widget = widget.widgets[0]
         attrs = sub_widget.build_attrs()
@@ -377,3 +460,11 @@ class AutoComboboxSelectMultipleWidgetTestCase(BaseWidgetTestCase):
         query = parse.query
         self.assertEqual(query, urlencode(params))
 
+    def test_build_selectable_options(self):
+        "Serialize selectable options as json in data attribute."
+        options = {'autoFocus': True}
+        widget = self.get_widget_instance(attrs={'data-selectable-options': options})
+        sub_widget = widget.widgets[0]
+        attrs = sub_widget.build_attrs()
+        self.assertTrue('data-selectable-options' in attrs)
+        self.assertEqual(attrs['data-selectable-options'], json.dumps(options))
