@@ -1,6 +1,6 @@
-from django.core.serializers import json
+from __future__ import unicode_literals
+
 from django.http import HttpResponse, Http404
-from django.utils import simplejson
 
 from selectable.registry import registry
 
@@ -9,7 +9,7 @@ def get_lookup(request, lookup_name):
 
     lookup_cls = registry.get(lookup_name)
     if lookup_cls is None:
-        raise Http404(u'Lookup %s not found' % lookup_name)
+        raise Http404('Lookup %s not found' % lookup_name)
 
     lookup = lookup_cls()
     return lookup.results(request)
